/*
 * TypeNameMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;

import com.topcoder.util.config.Property;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;


/**
 * <p>
 * Concrete implementation of the FileTypeMatcher class that matches file types with one or more type names. The type
 * name of the file type is configured in Magic Numbers component. The matching is case-sensitive.
 * </p>
 * 
 * <p>
 * To follow the reflection contract, this class can be constructed via a Property instance. The supported property is
 * named "types" which is potentially multi-valued.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcher extends FileTypeMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file type names to match. Empty file type names are not allowed even if
     * they are valid in the Magic Numbers component.
     * </p>
     */
    public static final String TYPES_PROPERTY_KEY = "types";

    /**
     * <p>
     * A list of file type names (String) to match. This list is initialized in constructor and is immutable.
     * </p>
     * 
     * <p>
     * To improve efficiency, the list is finally sorted.
     * </p>
     */
    private List types = null;

    /**
     * <p>
     * Constructor with single file type name. The type name cannot be empty.
     * </p>
     *
     * @param type the single file type name to match
     *
     * @throws NullPointerException if type is null
     * @throws IllegalArgumentException if type is empty
     */
    public TypeNameMatcher(String type) {
        if (type == null) {
            throw new NullPointerException("type is null");
        }
        if (type.trim().length() == 0) {
            throw new IllegalArgumentException("type is empty");
        }

        // create a List to hold the single type name
        types = new ArrayList();
        types.add(type);
    }

    /**
     * <p>
     * Constructor with multiple file type names. The type element cannot be empty String. The class will store a copy
     * of the list.
     * </p>
     *
     * @param types the multiple file type names to match
     *
     * @throws NullPointerException if types is null
     * @throws IllegalArgumentException if types is empty or contains non-String instance, or contains empty String
     */
    public TypeNameMatcher(List types) {
        MatcherUtil.checkListForString(types, false);

        // save the sorted types
        this.types = new ArrayList(types);
        Collections.sort(this.types);
    }

    /**
     * <p>
     * Constructor that loads types from a property. It should look for a "types" sub-property and extract multiple
     * values. The property value cannot be empty.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if the "types" sub-property is missing or it does not hold any values, or any
     *         property value is empty
     */
    public TypeNameMatcher(Property property) throws ConfigurationException {
        String[] types = MatcherUtil.getPropertyValues(property, TYPES_PROPERTY_KEY, false);

        // save the sorted types
        this.types = new ArrayList(Arrays.asList(types));
        Collections.sort(this.types);
    }

    /**
     * <p>
     * Checks whether the file type matches any of the file type names defined in the matcher. The matching is
     * case-sensitive.
     * </p>
     *
     * @param type the file type to match
     *
     * @return whether the file type matches
     */
    protected boolean match(FileType type) {
        String name = type.getName();

        // look for the file type name in the list in a case-sensitive way
        return Collections.binarySearch(types, name) >= 0;
    }
}
