/*
 * AbstractStatsReporting.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.io.File;
import java.io.IOException;


/**
 * <p>
 * This class provides a StatsReporting implementation base. It aggregates results from each processor and classifies
 * them based on single file, directory or alias. It also provides an overall statistical result. It defers the actual
 * rendering logic to concrete implementations.
 * </p>
 * 
 * <p>
 * If base directory is specified, the file paths in the file stats and directory paths in the directory stats are all
 * relative to the base directory. Otherwise, the paths will be absolute.
 * </p>
 * 
 * <p>
 * It does not need any configuration. However constructor is maintained to accept property argument for reflection
 * purpose.
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractStatsReporting implements StatsReporting {
    /**
     * <p>
     * The stats mapped to each file. The file path is relative to base directory or absolute if no base directory. Its
     * reference never changes.
     * </p>
     */
    private StatsCollection fileStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each directory. The directory path is relative to base directory or absolute if no base
     * directory. Its reference never changes.
     * </p>
     */
    private StatsCollection directoryStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each alias. Its reference never changes.
     * </p>
     */
    private StatsCollection aliasStats = new StatsCollection();

    /**
     * <p>
     * The overall statistical result. It is created anew when reporting resets.
     * </p>
     */
    private StatsResult overallStats = new StatsResult();

    /**
     * <p>
     * The base directory for the reporting. This variable is nullable (meaning all paths are absolute).
     * </p>
     */
    private File baseDirectory = null;

    /**
     * <p>
     * Default protected constructor to be accessed by subclass. Does nothing.
     * </p>
     */
    protected AbstractStatsReporting() {
    }

    /**
     * <p>
     * Protected constructor with property to be accessed by subclass. This does not have any functionality in the
     * current release and is kept for extension purpose.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if configuration error occurs
     */
    protected AbstractStatsReporting(Property property)
        throws ConfigurationException
    {
        if (property == null) {
            throw new NullPointerException("property is null");
        }
    }

    /**
     * <p>
     * Aggregates a result into the reporting against file, each path it is in, alias and the overall statistical
     * result.
     * </p>
     * 
     * <p>
     * If base directory is specified, the file path and directory path are relative to base directory. Otherwise, the
     * paths are absolute. Notice the directory stats does not need to include the base directory, since it's the same
     * as overall.
     * </p>
     *
     * @param processor the processor that generated the result
     * @param file the file that was processed
     * @param result the processing result
     *
     * @throws NullPointerException if any argument is null
     * @throws StatsReportingException if baseDirectory is not null but file is not in base directory, or if error
     *         occurs in aggregation
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result)
        throws StatsReportingException
    {
        if (processor == null) {
            throw new NullPointerException("processor is null");
        }
        if (file == null) {
            throw new NullPointerException("file is null");
        }
        if (result == null) {
            throw new NullPointerException("result is null");
        }

        // extract the canonical file path
        String filePath = getCanonicalPath(file);

        if (baseDirectory != null) {
            String basePath = getCanonicalPath(baseDirectory);

            // include separator character in the base path to be stripped
            if (!basePath.endsWith(File.separator)) {
                basePath += File.separator;
            }

            // check if file is in base directory
            if (!filePath.startsWith(basePath)) {
                throw new StatsReportingException("file " + filePath + " is not in base directory " + basePath);
            }

            // strip the base path off the file path
            filePath = filePath.substring(basePath.length());
        }

        // aggregate the file stats
        if (file.isFile()) {
            fileStats.add(filePath, result);
        }

        // aggregate the directory stats for each path it is in
        StringBuffer sb = new StringBuffer(filePath);
        int index = -1;

        while ((index = sb.lastIndexOf(File.separator)) >= 0) {
            String dirPath = sb.substring(0, index);

            // add back the separator character for root directory
            if ((baseDirectory == null) && (dirPath.indexOf(File.separator) < 0)) {
                dirPath += File.separator;
            }

            // include everything after the base directory as the path
            directoryStats.add(dirPath, result);

            // proceed with the parent directory
            sb.setLength(index);
        }

        // aggregate the alias stats
        aliasStats.add(processor.getAlias(), result);

        // aggregate the overall statistical result
        overallStats.aggregate(result);
    }

    /**
     * <p>
     * Resets the reporting such that all the results are cleared. Base directory can be null when absolute paths are
     * desirable.
     * </p>
     *
     * @param baseDirectory the base directory to use, can be null
     *
     * @throws IllegalArgumentException if baseDirectory is not null but is not a valid directory
     */
    public void reset(File baseDirectory) {
        if ((baseDirectory != null) && !baseDirectory.isDirectory()) {
            throw new IllegalArgumentException("baseDirectory is not a valid directory");
        }

        // clear the results
        fileStats.clear();
        directoryStats.clear();
        aliasStats.clear();
        overallStats = new StatsResult();

        // set the base directory
        this.baseDirectory = baseDirectory;
    }

    /**
     * <p>
     * Gets the file stats kept in this reporting.
     * </p>
     *
     * @return the file stats
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <p>
     * Gets the directory stats kept in this reporting.
     * </p>
     *
     * @return the directory stats
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <p>
     * Gets the alias stats kept in this reporting.
     * </p>
     *
     * @return the alias stats
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <p>
     * Gets the overall statistical result kept in this reporting. Returns a clone of the inner result.
     * </p>
     *
     * @return the overall statistical result
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <p>
     * Gets the base directory for the reporting. Can be null if all paths are absolute.
     * </p>
     *
     * @return the base directory, can be null
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }

    /**
     * <p>
     * Returns the canonical pathname string of the specified file. This method conveniently wraps any IOException into
     * StatsReportingException.
     * </p>
     *
     * @param file the file to get the canonical pathname string from
     *
     * @return the canonical pathname string
     *
     * @throws StatsReportingException if error occurs in getting the canonical pathname string
     */
    private String getCanonicalPath(File file) throws StatsReportingException {
        try {
            return file.getCanonicalPath();
        } catch (IOException e) {
            throw new StatsReportingException("Fails to get the canonical pathname string", e);
        }
    }
}
