/*
 * XmlStatsReporting.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.File;
import java.io.StringWriter;

import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


/**
 * <p>
 * XmlStatsReporting is a reporting implementation that renders XML reports. For readability pretty formatting is used
 * for the output. When multiple filenames, directory names or aliases are listed, alphabetical order is used
 * (case-insensitive). The display name of the metric will be used to render the element node. Here is an example:
 * <pre>
 * &lt;report&gt;
 *     &lt;files&gt;
 *         &lt;file name="a\p.txt"&gt;
 *             &lt;line&gt;6&lt;/line&gt;
 *         &lt;/file&gt;
 *         &lt;file name="a\q.java"&gt;
 *             &lt;line&gt;18&lt;/line&gt;
 *         &lt;/file&gt;
 *         &lt;file name="b\test.sql"&gt;
 *             &lt;line&gt;22&lt;/line&gt;
 *         &lt;/file&gt;
 *     &lt;/files&gt;
 *     &lt;directories&gt;
 *         &lt;directory name="a"&gt;
 *             &lt;line&gt;24&lt;/line&gt;
 *         &lt;/directory&gt;
 *         &lt;directory name="b"&gt;
 *             &lt;line&gt;22&lt;/line&gt;
 *         &lt;/directory&gt;
 *     &lt;/directories&gt;
 *     &lt;aliases&gt;
 *         &lt;alias name="Text File"&gt;
 *             &lt;line&gt;28&lt;/line&gt;
 *         &lt;/alias&gt;
 *         &lt;alias name="Source File"&gt;
 *             &lt;line&gt;18&lt;/line&gt;
 *         &lt;/alias&gt;
 *     &lt;/aliases&gt;
 *     &lt;overall&gt;
 *         &lt;line&gt;46&lt;/line&gt;
 *     &lt;/overall&gt;
 * &lt;/report&gt;
 * </pre>
 * </p>
 * 
 * <p>
 * It supports a single property "stylesheet" which contains a filename of the stylesheet to transform the XML into
 * user-oriented format (HTML, CSV, etc.). This property is optional. If it's missing the raw XML is generated.
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReporting extends AbstractStatsReporting {
    /**
     * <p>
     * The property which specifies a stylesheet to apply on the XML. If this property is not specified, the raw XML is
     * generated.
     * </p>
     */
    public static final String STYLESHEET_PROPERTY_KEY = "stylesheet";

    /**
     * <p>
     * The name of the key attribute in the stats element.
     * </p>
     */
    private static final String NAME_ATTRIBUTE = "name";

    /**
     * <p>
     * The tag name of the root element.
     * </p>
     */
    private static final String ROOT_TAG_NAME = "report";

    /**
     * <p>
     * The tag name of the file stats element.
     * </p>
     */
    private static final String FILE_TAG_NAME = "file";

    /**
     * <p>
     * The tag name of the file group element.
     * </p>
     */
    private static final String FILE_GROUP_TAG_NAME = "files";

    /**
     * <p>
     * The tag name of the directory stats element.
     * </p>
     */
    private static final String DIRECTORY_TAG_NAME = "directory";

    /**
     * <p>
     * The tag name of the directory group element.
     * </p>
     */
    private static final String DIRECTORY_GROUP_TAG_NAME = "directories";

    /**
     * <p>
     * The tag name of the alias stats element.
     * </p>
     */
    private static final String ALIAS_TAG_NAME = "alias";

    /**
     * <p>
     * The tag name of the alias group element.
     * </p>
     */
    private static final String ALIAS_GROUP_TAG_NAME = "aliases";

    /**
     * <p>
     * The tag name of the overall stats element.
     * </p>
     */
    private static final String OVERALL_TAG_NAME = "overall";

    /**
     * <p>
     * The value of the indent property set in the transformer.
     * </p>
     */
    private static final String INDENT_VALUE = "yes";

    /**
     * <p>
     * The property name of the indent amount set in the transformer.
     * </p>
     */
    private static final String INDENT_AMOUNT_PROPERTY_NAME = "{http://xml.apache.org/xslt}indent-amount";

    /**
     * <p>
     * The property value of the indent amount set in the transformer.
     * </p>
     */
    private static final String INDENT_AMOUNT_PROPERTY_VALUE = "4";

    /**
     * <p>
     * The stylesheet to apply on the raw XML generated. This is initialized in constructor and never changed. The
     * value can be null indicating no stylesheet is used.
     * </p>
     */
    private File stylesheet = null;

    /**
     * <p>
     * The builder to create documents for generating reports. It is lazily instantiated in the generateReport()
     * method.
     * </p>
     */
    private DocumentBuilder documentBuilder = null;

    /**
     * <p>
     * The factory to create transformers for generating reports. It is lazily instantiated in the generateReport()
     * method.
     * </p>
     */
    private TransformerFactory transformerFactory = null;

    /**
     * <p>
     * The templates to create transformers for generating reports. It is lazily instantiated in the generateReport()
     * method. It can be null if stylesheet is not specified.
     * </p>
     */
    private Templates templates = null;

    /**
     * <p>
     * Default constructor. Does nothing.
     * </p>
     */
    public XmlStatsReporting() {
    }

    /**
     * <p>
     * Constructor with a stylesheet to apply on the raw XML generated.
     * </p>
     *
     * @param stylesheet the stylesheet to apply
     *
     * @throws NullPointerException if stylesheet is null
     * @throws IllegalArgumentException if stylesheet is not a valid file
     */
    public XmlStatsReporting(File stylesheet) {
        if (stylesheet == null) {
            throw new NullPointerException("stylesheet is null");
        }
        if (!stylesheet.isFile()) {
            throw new IllegalArgumentException("stylesheet is not a valid file");
        }
        this.stylesheet = stylesheet;
    }

    /**
     * <p>
     * Configures from a property. Propagates to super constructor. After that, it reads a String property named
     * "stylesheet" which contains a filename of the stylesheet to transform the XML. This property is optional, but
     * should be an existing filename if exists.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if configuration error occurs
     */
    public XmlStatsReporting(Property property) throws ConfigurationException {
        super(property);

        // load the value from the property
        String value = property.getValue(STYLESHEET_PROPERTY_KEY);

        if (value != null) {
            // create a File object with the file path
            stylesheet = new File(value);

            // check if it is a valid file
            if (!stylesheet.isFile()) {
                throw new ConfigurationException("Value " + value + " for the property " + STYLESHEET_PROPERTY_KEY
                    + " is not a valid file");
            }
        }
    }

    /**
     * <p>
     * This generates an XML report if no stylesheet is specified, or outputs the result by transforming the XML with
     * the stylesheet. Refer to the class documentation for the XML report format.
     * </p>
     *
     * @return the generated report
     *
     * @throws StatsReportingException if error occurs in generating report
     */
    public String generateReport() throws StatsReportingException {
        // make sure everything is initialized
        init();

        // use the DOM for easy transformation
        Document document = documentBuilder.newDocument();

        // prepare the root element under the document
        Element root = document.createElement(ROOT_TAG_NAME);

        document.appendChild(root);

        // write the file stats under the root
        writeStatsCollection(root, FILE_GROUP_TAG_NAME, FILE_TAG_NAME, getFileStats());

        // write the directory stats under the root
        writeStatsCollection(root, DIRECTORY_GROUP_TAG_NAME, DIRECTORY_TAG_NAME, getDirectoryStats());

        // write the alias stats under the root
        writeStatsCollection(root, ALIAS_GROUP_TAG_NAME, ALIAS_TAG_NAME, getAliasStats());

        // write the overall stats under the root
        writeStatsResult(root, OVERALL_TAG_NAME, getOverallStats());

        // transform the document into a String
        return transform(document);
    }

    /**
     * <p>
     * Makes sure that the DocumentBuilder, TransformerFactory and Templates instances are created. They are created
     * once and are used to generate subsequent reports. This method also conveniently wraps the XML exceptions into
     * StatsReportingException.
     * </p>
     *
     * @throws StatsReportingException if the instances cannot be created
     */
    private void init() throws StatsReportingException {
        // lazily instantiate the DocumentBuilder
        if (documentBuilder == null) {
            try {
                documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            } catch (FactoryConfigurationError e) {
                throw new StatsReportingException("Fails to instantiate DocumentBuilderFactory", e);
            } catch (ParserConfigurationException e) {
                throw new StatsReportingException("Fails to instantiate DocumentBuilder", e);
            }
        }

        // lazily instantiate the TransformerFactory
        if (transformerFactory == null) {
            try {
                transformerFactory = TransformerFactory.newInstance();
            } catch (TransformerFactoryConfigurationError e) {
                throw new StatsReportingException("Fails to instantiate TransformerFactory", e);
            }
        }

        // lazily instantiate the Templates
        if ((stylesheet != null) && (templates == null)) {
            try {
                templates = transformerFactory.newTemplates(new StreamSource(stylesheet));
            } catch (TransformerConfigurationException e) {
                throw new StatsReportingException("Fails to instantiate Templates", e);
            }
        }
    }

    /**
     * <p>
     * Transforms the given document into a String. The stylesheet will be applied on the XML if it is specified.
     * Otherwise, the raw XML String will be returned. Pretty formatting is used in the transformation. This method
     * also conveniently wraps the XML exceptions into StatsReportingException.
     * </p>
     *
     * @param document the document to be transformed
     *
     * @return the transformed String
     *
     * @throws StatsReportingException if any error occurs in the transformation
     */
    private String transform(Document document) throws StatsReportingException {
        Transformer transformer = null;

        // instantiate the Transformer, using the stylesheet if specified
        try {
            if (stylesheet == null) {
                transformer = transformerFactory.newTransformer();
            } else {
                transformer = templates.newTransformer();
            }
        } catch (TransformerConfigurationException e) {
            throw new StatsReportingException("Fails to instantiate Transformer", e);
        }

        // make sure the output is pretty formatted
        transformer.setOutputProperty(OutputKeys.INDENT, INDENT_VALUE);
        transformer.setOutputProperty(INDENT_AMOUNT_PROPERTY_NAME, INDENT_AMOUNT_PROPERTY_VALUE);

        // create a StringWriter to store the resulting String
        StringWriter writer = new StringWriter();

        // do the transform
        try {
            transformer.transform(new DOMSource(document), new StreamResult(writer));
        } catch (TransformerException e) {
            throw new StatsReportingException("Fails to transform the XML", e);
        }

        // get back the String
        return writer.toString();
    }

    /**
     * <p>
     * Writes the group element of the given groupName under the given parent, and the stats elements of the given
     * statsName under the group element. Each stats element contains the metric elements with their values specified
     * in the collection.
     * </p>
     *
     * @param parent the parent to write to
     * @param groupName the name of the group element
     * @param statsName the name of the stats element
     * @param stats the statistical collection to write
     */
    private void writeStatsCollection(Element parent, String groupName, String statsName, StatsCollection stats) {
        Document document = parent.getOwnerDocument();

        // write the group element under the parent
        Element group = document.createElement(groupName);

        parent.appendChild(group);

        for (Iterator i = stats.getAllKeys().iterator(); i.hasNext();) {
            // write the stats element under the group
            Element child = document.createElement(statsName);

            group.appendChild(child);

            // set key as the stats attribute
            String key = (String) i.next();

            child.setAttribute(NAME_ATTRIBUTE, key);

            // write the value for each metric
            writeStatsResult(child, stats.get(key));
        }
    }

    /**
     * <p>
     * Writes the stats element of the given statsName under the given parent, and the metric elements with their
     * values specified in the result under the stats element.
     * </p>
     *
     * @param parent the parent to write to
     * @param statsName the name of the stats element
     * @param result the statistical result to write
     */
    private void writeStatsResult(Element parent, String statsName, StatsResult result) {
        Document document = parent.getOwnerDocument();

        // write the stats element under the parent
        Element child = document.createElement(statsName);

        parent.appendChild(child);

        // write the value for each metric
        writeStatsResult(child, result);
    }

    /**
     * <p>
     * Writes the metric elements with their values specified in the result under the given parent.
     * </p>
     *
     * @param parent the parent to write to
     * @param result the statistical result to write
     */
    private void writeStatsResult(Element parent, StatsResult result) {
        Document document = parent.getOwnerDocument();

        for (Iterator i = result.getMetrics().iterator(); i.hasNext();) {
            Metric metric = (Metric) i.next();
            String value = Long.toString(result.getStat(metric));

            // write the metric element under the parent
            Element child = document.createElement(metric.getDisplayName());

            parent.appendChild(child);

            // set value as the text content
            child.appendChild(document.createTextNode(value));
        }
    }
}
