/*
 * TypeNameMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.TypeNameMatcher;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * Unit tests for TypeNameMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcherTest extends TestCase {
    /**
     * The files to match against by TypeNameMatcher instances.
     */
    private static final String[] FILES = {"test_files/a/q.class", "test_files/b/test.xml", "test_files/b/test.sql"};

    /**
     * The TypeNameMatcher instances to test against.
     */
    private TypeNameMatcher[] matchers = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(TypeNameMatcherTest.class);

        return suite;
    }

    /**
     * Prepares three TypeNameMatcher instances for testing. matchers[0] is constructed with a single type name "Java
     * class file". matchers[1] is constructed with multiple type names "Java class file", "XML file". matchers[2] is
     * constructed with property loaded with type names "Java class file", "XML file".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        matchers = new TypeNameMatcher[3];
        matchers[0] = new TypeNameMatcher("Java class file");
        matchers[1] = new TypeNameMatcher(Arrays.asList(new String[] {"Java class file", "XML file"}));
        matchers[2] = new TypeNameMatcher(ConfigHelper.getRootProperty(ConfigHelper.GOOD_TYPE_NAME_MATCHER_NAMESPACE));
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null type. Expects NullPointerException.
     */
    public void testConstructor_NullType() {
        try {
            new TypeNameMatcher((String) null);
            fail("Creates TypeNameMatcher with null type");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty type. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyType() {
        try {
            new TypeNameMatcher("");
            fail("Creates TypeNameMatcher with empty type");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null types. Expects NullPointerException.
     */
    public void testConstructor_NullTypes() {
        try {
            new TypeNameMatcher((List) null);
            fail("Creates TypeNameMatcher with null types");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty types. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyTypes() {
        try {
            new TypeNameMatcher(new ArrayList());
            fail("Creates TypeNameMatcher with empty types");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with types containing non-String instance. Expects IllegalArgumentException.
     */
    public void testConstructor_NonStringTypes() {
        List types = new ArrayList();

        // add a non-String element
        types.add(new Object());

        try {
            new TypeNameMatcher(types);
            fail("Creates TypeNameMatcher with types containing non-String instance");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with types containing empty String. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyStringTypes() {
        List types = new ArrayList();

        // add an empty String
        types.add("");

        try {
            new TypeNameMatcher(types);
            fail("Creates TypeNameMatcher with types containing empty String");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new TypeNameMatcher((Property) null);
            fail("Creates TypeNameMatcher with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no value is present). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new TypeNameMatcher(ConfigHelper.getRootProperty(ConfigHelper.BAD_TYPE_NAME_MATCHER_NAMESPACE));
            fail("Creates TypeNameMatcher with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of match method with the three files. Expects results for matchers[0]: true, false, false. Expects results
     * for matchers[1]: true, true, false. Expects results for matchers[2]: true, true, false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch() throws Exception {
        File[] files = new File[FILES.length];

        for (int i = 0; i < FILES.length; i++) {
            files[i] = new File(FILES[i]);
        }

        // test with matchers[0] - only matches Java class file
        assertTrue("Fails to match file", matchers[0].match(files[0]));
        assertFalse("Fails to match file", matchers[0].match(files[1]));
        assertFalse("Fails to match file", matchers[0].match(files[2]));

        // test with matchers[1] - matches Java class file, XML file
        assertTrue("Fails to match file", matchers[1].match(files[0]));
        assertTrue("Fails to match file", matchers[1].match(files[1]));
        assertFalse("Fails to match file", matchers[1].match(files[2]));

        // test with matchers[2] - matches Java class file, XML file
        assertTrue("Fails to match file", matchers[2].match(files[0]));
        assertTrue("Fails to match file", matchers[2].match(files[1]));
        assertFalse("Fails to match file", matchers[2].match(files[2]));
    }
}
