/*
 * ExtendedFileStatistics.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;
import com.topcoder.file.TCSFile;


/**
 * <p>
 * This is the extended version of FileStatistics which supports some advanced operations including directory
 * processing and printing of the report. The reason to separate this out is that we may not need to rely on the
 * directory traversing algorithm incorporated here (which is recursive).
 * </p>
 *
 * <p>
 * This class shares the same configuration logic as FileStatistics. In addition to that, these properties are
 * supported since 2.0c:
 *
 * <ul>
 * <li>
 * "extractors" as a container for one or more sub properties that each specify an ArchiveExtractor
 * implementation. The sub property also contains the information for the ArchiveExtractor to configure from.
 * This is optional.
 * </li>
 * <li>
 * "temp_path" specifies the temp path to extract the temporary files to (and will be removed after the
 * processing is done). This is optional. And if it's not specified the default (system) temp path will be used.
 * </li>
 * </ul>
 * </p>
 *
 * <p>
 * Since version 2.0c archive extracting is supported. It supports a list of ArchiveExtractors registered to work with
 * certain type of archive file.
 * </p>
 *
 * <p>
 * All the operations are delegated. This class does not have any states and is fully dependent on FileStatistics
 * threading level.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 */
public class ExtendedFileStatistics extends FileStatistics {
    /**
     * <p>
     * The property which is a container for a list of extractors. Each sub property should have another subproperty
     * "classname" to specify the ArchiveExtractor. It is optional.
     * </p>
     *
     * @since 2.0c
     */
    public static final String EXTRACTORS_PROPERTY_KEY = "extractors";
    
    /**
     * <p>
     * This property specifies the fully qualified class name of the ArchiveExtractor implementation. The class should
     * have a public constructor that accepts a Property. It is required.
     * </p>
     *
     * @since 2.0c
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";
    
    /**
     * <p>
     * This property specifies the temp path to use in extracting. It's optional, but if exists, it should be a valid
     * directory.
     * </p>
     *
     * @since 2.0c
     */
    public static final String TEMP_PATH_PROPERTY_KEY = "temp_path";
    
    /**
     * <p>
     * A list of ArchiveExtractor instances used to extract archive files. The ArchiveExtractors are tried in order
     * and the first matching one is used. If none is applicable the file is processed as a normal file.
     * </p>
     *
     * @since 2.0c
     */
    private List extractors = new ArrayList();
    
    /**
     * <p>
     * The temp path used to extract the archive files. This can be null and if it is null the default temp path is used.
     * </p>
     *
     * @since 2.0c
     */
    private File tempPath = null;
    
    /**
     * <p>
     * The randomizer to generate random directory names inside the temp path.
     * </p>
     */
    private static final Random random = new Random();
    
    /**
     * <p>
     * Default constructor. Simply propagates to super constructor.
     * </p>
     */
    public ExtendedFileStatistics() {
        super();
    }
    
    /**
     * <p>
     * Constructor to load configuration from given namespace. Propagates to super constructor. Then loads the
     * extractors as well as the temp paths from configuration.
     * </p>
     *
     * @param namespace the namespace to load from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty
     * @throws ConfigurationException if configuration error occurs
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
        
        Property root = getRootProperty(namespace);
        
        // load the list of extractors (optional)
        Property extractors = root.getProperty(EXTRACTORS_PROPERTY_KEY);
        
        if (extractors != null) {
            for (Iterator i = extractors.list().iterator(); i.hasNext(); ) {
                Property extractor = (Property) i.next();
                
                addArchiveExtractor((ArchiveExtractor) configPropertyObject(extractor, CLASSNAME_PROPERTY_KEY,
                        ArchiveExtractor.class));
            }
        }
        
        // load the temp path (optional)
        String tempPath = root.getValue(TEMP_PATH_PROPERTY_KEY);
        
        if (tempPath != null) {
            try {
                setTempPath(new File(tempPath));
            } catch (IllegalArgumentException e) {
                throw new ConfigurationException(e.getMessage());
            }
        }
    }
    
    /**
     * <p>
     * Processes a directory in non-recursive mode. This is equivalent to calling processDirectory(directory, false).
     * </p>
     *
     * @param directory the directory to process
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     */
    public void processDirectory(File directory) throws IOException, FileStatisticsException {
        processDirectory(directory, false);
    }
    
    /**
     * <p>
     * Processes a directory in the given mode. If it's non-recursive only the files in the directory are processed.
     * Otherwise the files in all the sub-directories are processed.
     * </p>
     *
     * <p>
     * Since version 2.0c if recursive is on it delegates to processDirectory(directory, Integer.MAX_VALUE) so that zip
     * file processing is supported for an infinite nesting level. Otherwise zip file processing is turned off.
     * </p>
     *
     * @param directory the directory to process
     * @param recursive whether to process the directory recursively
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     */
    public void processDirectory(File directory, boolean recursive)
    throws IOException, FileStatisticsException {
        if (directory == null) {
            throw new NullPointerException("directory is null");
        }
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("directory is not a valid directory");
        }
        
        if (!recursive) {
            // if not recursive, process the files under the directory as normal
            File[] files = directory.listFiles();
            
            for (int i = 0; i < files.length; i++) {
                if (files[i].isFile()) {
                    processFile(files[i]);
                }
            }
        } else {
            // otherwise, process the directory recursively with zip file processing turned on
            processDirectory(directory, Integer.MAX_VALUE);
        }
    }
    
    /**
     * <p>
     * Processes a directory recursively with zip file processing turned on. The nesting level of archives should be
     * specified. The virtual path will just be the directory's real path.
     * </p>
     *
     * <p>
     * This is equivalent to calling processDirectory(directory, directory.path, nestingLevel).
     *
     * @param directory the directory to process
     * @param nestingLevel the maximum nesting level
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if directory is not a valid directory, or nestingLevel is negative
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     * @since 2.0c
     */
    public void processDirectory(File directory, int nestingLevel) throws IOException, FileStatisticsException {
        if (directory == null) {
            throw new NullPointerException("directory is null");
        }
        processDirectory(directory, directory.getCanonicalPath(), nestingLevel);
    }
    
    /**
     * <p>
     * Processes a directory recursively with zip file processing turned on. The virtual path to aggregate to and the
     * nesting level of archives should be specified. The temp path will be used for the extracted files, and will
     * be cleaned up after the processing is finished.
     * </p>
     *
     * @param directory the directory to process
     * @param virtualPath the virtual path to aggregate to
     * @param nestingLevel the maximum nesting level
     *
     * @throws NullPointerException if directory or virtualPath is null
     * @throws IllegalArgumentException if directory is not a valid directory, virtualPath is empty, or nestingLevel is negative
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the directory
     * @since 2.0c
     */
    protected void processDirectory(File directory, String virtualPath, int nestingLevel)
    throws IOException, FileStatisticsException {
        if (directory == null) {
            throw new NullPointerException("directory is null");
        }
        if (virtualPath == null) {
            throw new NullPointerException("virtualPath is null");
        }
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("directory is not a valid directory");
        }
        if (virtualPath.trim().length() == 0) {
            throw new IllegalArgumentException("virtualPath is empty");
        }
        if (nestingLevel < 0) {
            throw new IllegalArgumentException("nestingLevel is negative");
        }
        
        File[] files = directory.listFiles();
        
        for (int i = 0; i < files.length; i++) {
            String path = virtualPath + files[i].getName();
            
            if (files[i].isDirectory()) {
                // if it's a directory, recursively process it
                // Virtual path is appended by the path separator
                processDirectory(files[i], path + File.separator, nestingLevel);
            } else {
                ArchiveExtractor extractor = getArchiveExtractor(files[i]);
                
                if (extractor == null || nestingLevel == 0) {
                    // if an extractor is not applicable, process it as normal file
                    processFile(files[i], path);
                } else {
                    // otherwise, process it as an archive file
                    // Virtual path is appended by the path separator of the extractor
                    processArchiveFile(extractor, files[i], path + extractor.getPathSeparator(), nestingLevel - 1);
                }
            }
        }
    }
    
    /**
     * <p>
     * Processes the specified archive file by extracting it to a temporary directory (of a random name). Note that the
     * virtual path is passed unchanged for the directory processing. It will be cleaned up after processing.
     *
     * @param extractor the extractor the extract the archive file
     * @param archive the archive file to be extracted
     * @param virtualPath the virtual path to aggregate to
     * @param nestingLevel the maximum nesting level
     *
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the archive file
     */
    private void processArchiveFile(ArchiveExtractor extractor, File archive, String virtualPath, int nestingLevel)
    throws IOException, FileStatisticsException {
        File tempPath = this.tempPath;
        
        // use system-dependent temporary directory if temp path is not specified
        if (tempPath == null) {
            tempPath = new File(System.getProperty("java.io.tmpdir"));
        }
        
        File tempArchivePath = new File(tempPath, String.valueOf(random.nextInt()));
        
        // make a temp directory for the extractor
        tempArchivePath.mkdir();
        
        // extract the archive file into the temp directory
        extractor.extract(archive, tempArchivePath);
        
        // process the temp directory with the current virtual path
        processDirectory(tempArchivePath, virtualPath, nestingLevel);
        
        // cleanup the temp directory after processing
        new TCSFile(tempArchivePath).deleteTree();
    }
    
    /**
     * <p>
     * Determines the archive extractor for the specified file. If it is not an archive, null is returned.
     * </p>
     *
     * @param file the file to check
     * @return the archive extractor for the specified file, or null if there is none
     * @throws IOException if I/O operation fails
     * @throws FileMatchingException if any error occurs in matching the extractor
     */
    private ArchiveExtractor getArchiveExtractor(File file) throws IOException, FileMatchingException {
        // look for a matching archive extractor
        for (Iterator i = extractors.iterator(); i.hasNext(); ) {
            ArchiveExtractor extractor = (ArchiveExtractor) i.next();
            
            if (extractor.getFileMatcher().match(file)) {
                return extractor;
            }
        }
        
        // all extractors have been exhausted
        return null;
    }
    
    /**
     * <p>
     * Prints the report to console.
     * </p>
     *
     * @throws StatsReportingException if error occurs in generating report
     */
    public void printReport() throws StatsReportingException {
        System.out.print(getReporting().generateReport());
    }
    
    /**
     * <p>
     * Prints the report to the given file.
     * </p>
     *
     * @param file the file to write to
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     * @throws StatsReportingException if error occurs in generating report
     */
    public void printReport(File file) throws IOException, StatsReportingException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }
        
        Writer out = null;
        
        try {
            // create a BufferedWriter to write a String
            out = new BufferedWriter(new FileWriter(file));
            
            // write the report to the file
            out.write(getReporting().generateReport());
        } finally {
            close(out);
        }
    }
    
    /**
     * <p>
     * Adds the extractor to the end of the list (do not care about duplication).
     * </p>
     *
     *
     * @return the position of the added extractor
     * @param extractor the extractor to add
     * @throws NullPointerException if extractor is null
     * @since 2.0c
     */
    public int addArchiveExtractor(ArchiveExtractor extractor) {
        if (extractor == null) {
            throw new NullPointerException("extractor is null");
        }
        extractors.add(extractor);
        return extractors.size() - 1;
    }
    
    /**
     * <p>
     * Removes the extractor from given position.
     * </p>
     *
     *
     * @return the removed extractor
     * @param index the index of the extractor to remove
     * @throws IndexOutOfBoundsException if index is not valid
     * @since 2.0c
     */
    public ArchiveExtractor removeArchiveExtractor(int index) {
        return (ArchiveExtractor) extractors.remove(index);
    }
    
    /**
     * <p>
     * Gets a shallow copy of the extractor list.
     * </p>
     *
     *
     * @return a list of extractors
     */
    public List getAllArchiveExtractors() {
        return new ArrayList(extractors);
    }
    
    /**
     * <p>
     * Clears all the extractors.
     * </p>
     *
     *
     * @since 2.0c
     */
    public void clearAllArchiveExtractors() {
        extractors.clear();
    }
    
    /**
     * <p>
     * Gets the temp path used to extract the archive files. This can be null.
     * </p>
     *
     *
     * @return the temp path, can be null.
     * @since 2.0c
     */
    public File getTempPath() {
        return tempPath;
    }
    
    /**
     * <p>
     * Sets the temp path used to extract the archive files. It can be null to indicate the default temp path is used.
     * </p>
     *
     *
     * @param tempPath the temp path to set, can be null.
     * @throws IllegalArgumentException if tempPath is not a valid directory.
     * @since 2.0c
     */
    public void setTempPath(File tempPath) {
        if (tempPath != null && !tempPath.isDirectory()) {
            throw new IllegalArgumentException("tempPath is not a valid directory");
        }
        this.tempPath = tempPath;
    }
    
    /**
     * <p>
     * Instantiates the class based on the information collected from the specified property, using a Property
     * argument. The class name is specified under the property of the given name. The property should be a node
     * representing the class in the configuration file. The class to be instantiated must be of type clazz.
     * </p>
     *
     * @param property a node representing the class.
     * @param name the name of the property specifying the class name.
     * @param clazz the superclass or superinterface of the instantiated object.
     *
     * @return an instance of the specified class.
     *
     * @throws ConfigurationException if a problem occurs while configuring, or required properties are missing or
     *         empty.
     */
    private Object configPropertyObject(Property property, String name, Class clazz)
    throws ConfigurationException {
        String className = getPropertyValue(property, name);
        
        // locate the class for instantiation
        Class objClazz = null;
        
        try {
            objClazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new ConfigurationException("Unable to locate the class " + className, e);
        }
        
        if (!clazz.isAssignableFrom(objClazz)) {
            throw new ConfigurationException("Class " + className + " is not of type " + clazz.getName());
        }
        
        // instantiate the class with a Property argument
        Class[] parameterTypes = new Class[] {Property.class};
        Object[] parameters = new Object[] {property};
        
        try {
            return objClazz.getConstructor(parameterTypes).newInstance(parameters);
        } catch (Exception e) {
            throw new ConfigurationException("Fails to create " + className + " instance", e);
        }
    }
    
    /**
     * <p>
     * Gets the property value of the specified name. If the property value does not exist or is empty, a
     * ConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name.
     *
     * @throws ConfigurationException if the property value does not exist or is empty.
     */
    private String getPropertyValue(Property property, String name)
    throws ConfigurationException {
        String value = property.getValue(name);
        
        if (value == null) {
            throw new ConfigurationException("Missing value for property " + name);
        }
        if (value.trim().length() == 0) {
            throw new ConfigurationException("Empty value for property " + name);
        }
        return value;
    }
    
    /**
     * <p>
     * Obtains the root property of the configuration file in the specified namespace. This method conveniently wraps
     * the exceptions from the Configuration Manager into ConfigurationException.
     * </p>
     *
     * @param namespace the configuration namespace to use.
     *
     * @return the root property of the configuration file.
     *
     * @throws ConfigurationException if a problem occurs while configuring.
     */
    private Property getRootProperty(String namespace)
    throws ConfigurationException {
        try {
            return ConfigManager.getInstance().getPropertyObject(namespace, "");
        } catch (ConfigManagerException e) {
            throw new ConfigurationException("Fails to load from configuration namespace " + namespace, e);
        }
    }
    
    /**
     * <p>
     * Try to close the writer object if it is not null. Returns silently if any I/O error occurs.
     * </p>
     *
     * @param writer the writer object.
     */
    private static void close(Writer writer) {
        if (writer != null) {
            try {
                writer.close();
            } catch (IOException e) {}
        }
    }
}
