/*
 * FileProcessor.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.IOException;

import java.util.List;


/**
 * <p>
 * FileProcessor defines the contract to process a file. One file processor defines an algorithm to produce statistical
 * result for a certain class of files against certain metrics. Multiple FileProcessors are aggregated into
 * FileStatistics, which can run in two modes (process with first matching processor and process with each matching
 * processor). Each file processor is mapped with an alias. This alias is dynamically provided by the user and the
 * reporting can be grouped against this alias.
 * </p>
 * 
 * <p>
 * All implementations should have a constructor with a Property as an argument in order to reflect from configuration
 * file. This requirement can be dropped if it's not necessary to use this way.
 * </p>
 * 
 * <p>
 * Implementations are advised to be stateless and thread-safe. But it's not a requirement.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public interface FileProcessor {
    /**
     * <p>
     * Gets a list of all the supported metrics by this processor. This method is mostly for end user. It does not
     * fulfill any responsibility in the current framework.
     * </p>
     *
     * @return a list of Metric instances
     */
    List getSupportedMetrics();

    /**
     * <p>
     * Gets the alias of the processor. An alias is the official name of the processor which is also used to classify
     * the statistical result against. This should be non-empty.
     * </p>
     *
     * @return the alias of the file processor
     */
    String getAlias();

    /**
     * <p>
     * Gets the file matcher associated with the processor in order to determine whether a file should be processed
     * with the processor.
     * </p>
     *
     * @return a file matcher associated with the processor
     */
    FileMatcher getFileMatcher();

    /**
     * <p>
     * Processes the file and generates the result. A new result should be generated each time, whose keys are the
     * metrics returned from getSupportedMetrics().
     * </p>
     *
     * @param file the file to process
     *
     * @return the statistical result
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     * @throws FileProcessingException if error occurs in file processing
     */
    StatsResult process(File file) throws IOException, FileProcessingException;
}
