/*
 * FileStatistics.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


/**
 * <p>
 * This is the main class of the user API. It provides the basic API, which includes maintaining a list of file
 * processors, a reporting handler and controlling the way to process the file. Files are processed one by one and the
 * results are aggregated into the reporting. If you want to start a new reporting you should either explicitly set
 * the reporting or get the old reporting to reset it.
 * </p>
 *
 * <p>
 * There are two ways to process the file:
 *
 * <ul>
 * <li>
 * Process it with the first matching processor.
 * </li>
 * <li>
 * Process it with each matching processor.
 * </li>
 * </ul>
 *
 * Refer to ExtendedFileStatistics for advanced usage.
 * </p>
 *
 * <p>
 * This class can be initialized from a configuration namespace which should be preloaded. It will look for the
 * following properties:
 *
 * <ul>
 * <li>
 * "processors" as a container for file processors, each sub-property should contain a "classname" that specifies the
 * fully qualified class name of a FileProcessor implementation which accepts a Property as constructor argument. This
 * property is required.
 * </li>
 * <li>
 * "reporting" which contains a "classname" that specifies the fully qualified class name of a StatsReporting
 * implementation which accepts a Property as constructor argument. This property is optional (XmlStatsReporting is
 * the default value).
 * </li>
 * <li>
 * "matchall" which is a boolean flag that indicates whether all matching processors will be used to process the file.
 * It is regarded true when value is either "on", "yes", "true" case-insensitively. This property is optional (default
 * value is false).
 * </li>
 * </ul>
 * </p>
 *
 * <p>
 * This class is not thread-safe in that it provides a number of manipulation methods in a non-synchronized manner.
 * Even if calling from different threads at separated time will mix the result into the same reporting, which may not
 * be the desirable behavior.
 * </p>
 *
 * <p>
 * Since version 2.0c there is a minor fix that adds a method to the FileStatistics to aggregate according to
 * given virtual path rather than the real file path.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class FileStatistics {
    /**
     * <p>
     * This property contains multiple sub-properties to specify FileProcessor implementations. When the implementation
     * is instantiated the property itself is passed in as constructor argument. This property is required.
     * </p>
     */
    public static final String PROCESSORS_PROPERTY_KEY = "processors";
    
    /**
     * <p>
     * This property contains sub-properties to specify the StatsReporting implementation. When the implementation is
     * instantiated the property itself is passed in as constructor argument. This property is optional
     * (XmlStatsReporting is the default value)
     * </p>
     */
    public static final String REPORTING_PROPERTY_KEY = "reporting";
    
    /**
     * <p>
     * This property specifies whether a file is processed by the first matching processor or each of the matching
     * processors. This property is optional (default to false). Its value is considered true if it's "true", "yes" or
     * "on" (case-insensitive).
     * </p>
     */
    public static final String MATCH_ALL_PROPERTY_KEY = "matchall";
    
    /**
     * <p>
     * This is a sub-property in "processors" sub-properties or "reporting" to specify the fully qualified class name
     * of the FileProcessor or StatsReporting implementation that owns a constructor with a Property argument.
     * </p>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";
    
    /**
     * <p>
     * The values (case-insensitive) for the "matchall" property indicating that the matchall flag is true.
     * </p>
     */
    private static final String[] MATCH_ALL_VALUES = {"on", "yes", "true"};
    
    /**
     * <p>
     * The processors aggregated. Each element in the list is a FileProcessor instance. This is initialized in
     * constructor and the reference will never change. However the list contents can be changed in the add/remove
     * methods.
     * </p>
     */
    private List processors = null;
    
    /**
     * <p>
     * The reporting to use. This is initialized in constructor and cannot be null. This is mutable by the setter
     * method.
     * </p>
     */
    private StatsReporting reporting = null;
    
    /**
     * <p>
     * A flag indicating whether to process the file with each matching processor. This is mutable by the setter
     * method.
     * </p>
     */
    private boolean matchAll = false;
    
    /**
     * <p>
     * Default constructor. The list of processors is empty, the reporting is set to XmlStatsReporting, and the
     * matchall flag is set to false.
     * </p>
     */
    public FileStatistics() {
        processors = new LinkedList();
        setReporting(new XmlStatsReporting());
        setMatchAll(false);
    }
    
    /**
     * <p>
     * Constructor to load configuration from given namespace. The namespace should be already loaded. It loads the
     * list of processors, the reporting as well as the matchall flag.
     * </p>
     *
     * @param namespace the namespace to load from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty
     * @throws ConfigurationException if configuration error occurs
     */
    public FileStatistics(String namespace) throws ConfigurationException {
        this(); // set everything to default values first
        
        checkString(namespace);
        
        Property root = getRootProperty(namespace);
        
        // load the list of processors (required)
        Property processors = root.getProperty(PROCESSORS_PROPERTY_KEY);
        
        if (processors == null) {
            throw new ConfigurationException("Missing property " + PROCESSORS_PROPERTY_KEY);
        }
        
        for (Iterator i = processors.list().iterator(); i.hasNext();) {
            Property processor = (Property) i.next();
            
            addFileProcessor((FileProcessor) configPropertyObject(processor, CLASSNAME_PROPERTY_KEY, FileProcessor.class));
        }
        
        // load the reporting (optional)
        Property reporting = root.getProperty(REPORTING_PROPERTY_KEY);
        
        if (reporting != null) {
            setReporting((StatsReporting) configPropertyObject(reporting, CLASSNAME_PROPERTY_KEY, StatsReporting.class));
        }
        
        // load the matchall flag (optional)
        String value = root.getValue(MATCH_ALL_PROPERTY_KEY);
        
        if (value != null) {
            for (int i = 0; i < MATCH_ALL_VALUES.length; i++) {
                if (value.equalsIgnoreCase(MATCH_ALL_VALUES[i])) {
                    setMatchAll(true);
                    break;
                }
            }
        }
    }
    
    /**
     * <p>
     * Adds the file processor to the aggregation. If there is a processor with the same alias the old processor is
     * removed. The new processor is always added to the end of the list.
     * </p>
     *
     * @param processor the file processor to add
     *
     * @throws NullPointerException if processor is null
     */
    public void addFileProcessor(FileProcessor processor) {
        if (processor == null) {
            throw new NullPointerException("processor is null");
        }
        
        // remove the old processor
        removeFileProcessor(processor.getAlias());
        
        // add the new processor to the end
        processors.add(processor);
    }
    
    /**
     * <p>
     * Removes the file processor with given alias. If no such processor exists, it returns null.
     * </p>
     *
     * @param alias the alias of the processor
     *
     * @return the processor just removed, or null if no such processor exists
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty
     */
    public FileProcessor removeFileProcessor(String alias) {
        return getFileProcessor(alias, true);
    }
    
    /**
     * <p>
     * Gets the file processor with given alias. If no such processor exists, it returns null.
     * </p>
     *
     * @param alias the alias of the processor
     *
     * @return the associated file processor, or null if no such processor exists
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty
     */
    public FileProcessor getFileProcessor(String alias) {
        return getFileProcessor(alias, false);
    }
    
    /**
     * <p>
     * Gets a list of all the processors aggregated. Should return a copy of the inner collection.
     * </p>
     *
     * @return a list of all the processors
     */
    public List getAllFileProcessors() {
        return new ArrayList(processors);
    }
    
    /**
     * <p>
     * Clears all the aggregated processors.
     * </p>
     */
    public void clearAllFileProcessors() {
        processors.clear();
    }
    
    /**
     * <p>
     * Gets the stats reporting implementation, must not be null.
     * </p>
     *
     * @return the stats reporting implementation
     */
    public StatsReporting getReporting() {
        return reporting;
    }
    
    /**
     * <p>
     * Sets the stats reporting implementation. Once the new instance is set the old results are dropped.
     * </p>
     *
     * @param reporting the stats reporting implementation
     *
     * @throws NullPointerException if reporting is null
     */
    public void setReporting(StatsReporting reporting) {
        if (reporting == null) {
            throw new NullPointerException("reporting is null");
        }
        this.reporting = reporting;
    }
    
    /**
     * <p>
     * Returns whether to process the file with each matching processor.
     * </p>
     *
     * @return whether to process the file with each matching processor
     */
    public boolean isMatchAll() {
        return matchAll;
    }
    
    /**
     * <p>
     * Sets whether to process the file with each matching processor.
     * </p>
     *
     * @param matchAll whether to process the file with each matching processor
     */
    public void setMatchAll(boolean matchAll) {
        this.matchAll = matchAll;
    }
    
    /**
     * <p>
     * Processes a single file. This method does:
     *
     * <ul>
     * <li>
     * If matchall is on, iterate through all the file processors.
     * </li>
     * <li>
     * If matchall is off, iterate until we meet the first matching processor.
     * </li>
     * <li>
     * Process the file with the matched processor(s).
     * </li>
     * <li>
     * Aggregate the result into the reporting.
     * </li>
     * <li>
     * If no processor matches the file, nothing happens.
     * </li>
     * </ul>
     * </p>
     *
     * <p>
     * This is equivalent to calling the processFile(file, virtualPath) method with the file's real path as the
     * virtual path.
     * </p>
     *
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the file
     */
    public void processFile(File file) throws IOException, FileStatisticsException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }
        processFile(file, file.getCanonicalPath());
    }
    
    /**
     * <p>
     * This method is added since version 2.0c. It should take over the logic of the original processFile() method.
     * The difference is that the result is aggregated with virtual path rather than the file's real path.
     * </p>
     *
     *
     * @param file the file to process
     * @param virtualPath the virtual path for the file
     * @throws NullPointerException if file or virtualPath is null
     * @throws IllegalArgumentExcpetion if virtualPath is empty
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in processing the file
     *
     * @since 2.0c
     */
    protected void processFile(File file, String virtualPath) throws IOException, FileStatisticsException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }
        if (virtualPath == null) {
            throw new NullPointerException("virtualPath is null");
        }
        if (virtualPath.trim().length() == 0) {
            throw new IllegalArgumentException("virtualPath is empty");
        }
        
        // match the file against each file processor
        for (Iterator i = processors.iterator(); i.hasNext();) {
            FileProcessor processor = (FileProcessor) i.next();
            
            if (processor.getFileMatcher().match(file)) {
                // if the file matches, process the file
                StatsResult result = processor.process(file);
                
                // aggregate the result into the reporting
                reporting.aggregate(processor, virtualPath, result);
                
                // if matchall is off, stop the file from further processing
                if (!matchAll) {
                    break;
                }
            }
        }
    }
    
    /**
     * <p>
     * Instantiates the class based on the information collected from the specified property, using a Property
     * argument. The class name is specified under the property of the given name. The property should be a node
     * representing the class in the configuration file. The class to be instantiated must be of type clazz.
     * </p>
     *
     * @param property a node representing the class.
     * @param name the name of the property specifying the class name.
     * @param clazz the superclass or superinterface of the instantiated object.
     *
     * @return an instance of the specified class.
     *
     * @throws ConfigurationException if a problem occurs while configuring, or required properties are missing or
     *         empty.
     */
    private Object configPropertyObject(Property property, String name, Class clazz)
    throws ConfigurationException {
        String className = getPropertyValue(property, name);
        
        // locate the class for instantiation
        Class objClazz = null;
        
        try {
            objClazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new ConfigurationException("Unable to locate the class " + className, e);
        }
        
        if (!clazz.isAssignableFrom(objClazz)) {
            throw new ConfigurationException("Class " + className + " is not of type " + clazz.getName());
        }
        
        // instantiate the class with a Property argument
        Class[] parameterTypes = new Class[] {Property.class};
        Object[] parameters = new Object[] {property};
        
        try {
            return objClazz.getConstructor(parameterTypes).newInstance(parameters);
        } catch (Exception e) {
            throw new ConfigurationException("Fails to create " + className + " instance", e);
        }
    }
    
    /**
     * <p>
     * Gets the property value of the specified name. If the property value does not exist or is empty, a
     * ConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name.
     *
     * @throws ConfigurationException if the property value does not exist or is empty.
     */
    private String getPropertyValue(Property property, String name)
    throws ConfigurationException {
        String value = property.getValue(name);
        
        if (value == null) {
            throw new ConfigurationException("Missing value for property " + name);
        }
        if (value.trim().length() == 0) {
            throw new ConfigurationException("Empty value for property " + name);
        }
        return value;
    }
    
    /**
     * <p>
     * Obtains the root property of the configuration file in the specified namespace. This method conveniently wraps
     * the exceptions from the Configuration Manager into ConfigurationException.
     * </p>
     *
     * @param namespace the configuration namespace to use.
     *
     * @return the root property of the configuration file.
     *
     * @throws ConfigurationException if a problem occurs while configuring.
     */
    private Property getRootProperty(String namespace)
    throws ConfigurationException {
        try {
            return ConfigManager.getInstance().getPropertyObject(namespace, "");
        } catch (ConfigManagerException e) {
            throw new ConfigurationException("Fails to load from configuration namespace " + namespace, e);
        }
    }
    
    /**
     * <p>
     * Gets the file processor with given alias. If the processor exists and remove flag is true, the processor is
     * removed from the list. If no such processor exists, it returns null.
     * </p>
     *
     * @param alias the alias of the processor
     * @param remove whether the associated file processor should be removed
     *
     * @return the associated file processor, or null if no such processor exists
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty
     */
    private FileProcessor getFileProcessor(String alias, boolean remove) {
        checkString(alias);
        
        // look for the processor with the alias
        for (Iterator i = processors.iterator(); i.hasNext();) {
            FileProcessor processor = (FileProcessor) i.next();
            
            if (alias.equals(processor.getAlias())) {
                // remove the processor from the list if remove flag is true
                if (remove) {
                    i.remove();
                }
                return processor;
            }
        }
        
        // all processors have been exhausted
        return null;
    }
    
    /**
     * <p>
     * Checks if the given string is null or empty.
     * </p>
     *
     * @param str the string to check against.
     *
     * @throws NullPointerException if str is null.
     * @throws IllegalArgumentException if str is empty.
     */
    private void checkString(String str) {
        if (str == null) {
            throw new NullPointerException("String argument is null");
        }
        if (str.trim().length() == 0) {
            throw new IllegalArgumentException("String argument is empty");
        }
    }
}
