/*
 * StatsResult.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * StatsResult is a container for multiple statistical results, each of which are mapped from metric (Metric) to the
 * statistical quantity (Long). This class wraps some map operations as well as provides a way to clone and aggregate
 * results.
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsResult {
    /**
     * <p>
     * The results mapping from metric (Metric) to the statistical quantity (Long). It is initialized in constructor
     * and the reference is never changed.
     * </p>
     */
    private Map results = null;

    /**
     * <p>
     * Default constructor. The result is empty.
     * </p>
     */
    public StatsResult() {
        results = new HashMap();
    }

    /**
     * <p>
     * Creates a result based on another result. This acts as a way to clone the instance.
     * </p>
     *
     * @param result the result to clone from
     *
     * @throws NullPointerException if result is null
     */
    public StatsResult(StatsResult result) {
        if (result == null) {
            throw new NullPointerException("result is null");
        }

        // clone the inner mapping
        results = new HashMap(result.results);
    }

    /**
     * <p>
     * Sets the metric with the given quantity. This method will overwrite old quantity associated with the metric, if
     * there is any.
     * </p>
     *
     * @param metric the metric to associate with
     * @param result the quantity as a long
     *
     * @throws NullPointerException if metric is null
     */
    public void setStat(Metric metric, long result) {
        if (metric == null) {
            throw new NullPointerException("metric is null");
        }
        results.put(metric, new Long(result));
    }

    /**
     * <p>
     * Gets the quantity associated with the given metric.
     * </p>
     *
     * @param metric the metric associated with quantity
     *
     * @return the quantity as a long
     *
     * @throws NullPointerException if metric is null
     * @throws IllegalArgumentException if no quantity is associated with the metric
     */
    public long getStat(Metric metric) {
        if (metric == null) {
            throw new NullPointerException("metric is null");
        }

        Long result = (Long) results.get(metric);

        // check if metric exists in the mapping
        if (result == null) {
            throw new IllegalArgumentException("No quantity is associated with the metric " + metric.getName());
        }

        return result.longValue();
    }

    /**
     * <p>
     * Gets a list of all the associated metrics. Should return a copy of the inner collection.
     * </p>
     *
     * @return a list of associated metrics
     */
    public List getMetrics() {
        return new ArrayList(results.keySet());
    }

    /**
     * <p>
     * Aggregates the given result into the current result for all the metrics. If the metric already exists, the old
     * quantity will be added with the quantity from the aggregating result. Otherwise, a new metric will be
     * associated.
     * </p>
     *
     * @param result the result to aggregate from
     *
     * @throws NullPointerException if result is null
     */
    public void aggregate(StatsResult result) {
        if (result == null) {
            throw new NullPointerException("result is null");
        }

        // aggregate the result for each metric
        for (Iterator i = result.results.entrySet().iterator(); i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            Metric metric = (Metric) entry.getKey();
            Long oldValue = (Long) results.get(metric);
            Long newValue = (Long) entry.getValue();

            // if old quantity exists, add the aggregating quantity to it
            if (oldValue != null) {
                newValue = new Long(oldValue.longValue() + newValue.longValue());
            }

            // put the new quantity to the mapping
            results.put(metric, newValue);
        }
    }
}
