/*
 * ArchiveExtractorBase.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.extractor;

import com.topcoder.file.statistics.ArchiveExtractor;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.util.config.Property;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.ConfigurationException;

/**
 * <p>
 * An implementation base for ArchiveExtractor which implements the logic to load configuration for the file
 * matcher and path separator.
 * </p>
 *
 * <p>
 * This class can load configuration from a Property. The following properties are supported:
 *
 * <ul>
 * <li>
 * "matcher" which specifies a file matcher. It contains a "classname" which specifies the fully qualified class name
 * of a FileMatcher implementation that owns a constructor with a Property argument. If not specified AnyFileMatcher
 * is used.
 * </li>
 * <li>
 * "path_separator" which contains single character to start the virtual path extracted from this extractor. If not
 * specified the "&gt;" character is used.
 * </li>
 * </ul>
 * </p>
 *
 * <p>
 * This class itself is thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 * @since 2.0c
 */
public abstract class ArchiveExtractorBase implements ArchiveExtractor {
    
    /**
     * <p>
     * This property contains sub-properties to specify the attached FileMatcher implementation. When the
     * implementation is instantiated the property itself is passed in as constructor argument. If not specified
     * AnyFileMatcher is used.
     * </p>
     */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";
    
    /**
     * <p>
     * This is a sub-property in "matcher" to specify the fully qualified class name of the FileMatcher implementation
     * that owns a constructor with a Property argument.
     * </p>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";
    
    /**
     * <p>
     * The property specifying the path separator to use with this ArchiveExtractor. It is optional (defaulting to '>'). If it exists it should have
     * a single character.
     * </p>
     */
    public static final String PATH_SEPARATOR_PROPERTY_KEY = "path_separator";
    
    /**
     * <p>
     * This is the file matcher for the archive extractor (and determines if a file can/should be processed). It is set in
     * constructor and never changed.
     * </p>
     */
    private FileMatcher matcher = null;
    
    /**
     * <p>
     * The path separator to use with the ArchiveExtractor. It is set in constructor and never changes.
     * </p>
     *
     */
    private char pathSeparator = '>';
    
    /**
     * <p>
     * Constructor with file matcher and path separator. It is protected for subclass to access.
     * </p>
     *
     *
     * @param matcher the file matcher
     * @param pathSeparator the path separator
     * @throws NullPointerException if matcher is null
     */
    protected  ArchiveExtractorBase(FileMatcher matcher, char pathSeparator) {
        if (matcher == null) {
            throw new NullPointerException("matcher is null");
        }
        this.matcher = matcher;
        this.pathSeparator = pathSeparator;
    }
    
    /**
     * <p>
     * This constructor loads configuration from the property. It is protected for subclass to access.
     * </p>
     *
     * <p>
     * Any configuration-related error will be wrapped in a
     * ConfigurationException.
     * </p>
     *
     * @param property the property to configure from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if any error occurs in configuring the extractor
     */
    protected  ArchiveExtractorBase(Property property) throws ConfigurationException {
        if (property == null) {
            throw new NullPointerException("property is null");
        }
        
        // matcher property is optional
        Property matcher = property.getProperty(FILE_MATCHER_PROPERTY_KEY);
        
        if (matcher != null) {
            // instantiate the FileMatcher class if specified
            this.matcher = (FileMatcher) configPropertyObject(matcher, CLASSNAME_PROPERTY_KEY, FileMatcher.class);
        } else {
            // otherwise, AnyFileMatcher is used
            this.matcher = new AnyFileMatcher();
        }
        
        // path separator is optional
        String pathSeparator = property.getValue(PATH_SEPARATOR_PROPERTY_KEY);
        
        if (pathSeparator != null) {
            if (pathSeparator.length() != 1) {
                throw new ConfigurationException("Single character expected for property " + PATH_SEPARATOR_PROPERTY_KEY);
            }
            this.pathSeparator = pathSeparator.charAt(0);
        }
    }
    
    /**
     * <p>
     * Gets the file matcher.
     * </p>
     *
     *
     * @return the file matcher
     */
    public FileMatcher getFileMatcher() {
        return matcher;
    }
    
    /**
     * <p>
     * Gets the path separator.
     * </p>
     *
     *
     * @return the path separator
     */
    public char getPathSeparator() {
        return pathSeparator;
    }
    
    /**
     * <p>
     * Instantiates the class based on the information collected from the specified property, using a Property
     * argument. The class name is specified under the property of the given name. The property should be a node
     * representing the class in the configuration file. The class to be instantiated must be of type clazz.
     * </p>
     *
     * @param property a node representing the class.
     * @param name the name of the property specifying the class name.
     * @param clazz the superclass or superinterface of the instantiated object.
     *
     * @return an instance of the specified class.
     *
     * @throws ConfigurationException if a problem occurs while configuring, or required properties are missing or
     *         empty.
     */
    private Object configPropertyObject(Property property, String name, Class clazz)
    throws ConfigurationException {
        String className = getPropertyValue(property, name);
        
        // locate the class for instantiation
        Class objClazz = null;
        
        try {
            objClazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new ConfigurationException("Unable to locate the class " + className, e);
        }
        
        if (!clazz.isAssignableFrom(objClazz)) {
            throw new ConfigurationException("Class " + className + " is not of type " + clazz.getName());
        }
        
        // instantiate the class with a Property argument
        Class[] parameterTypes = new Class[] {Property.class};
        Object[] parameters = new Object[] {property};
        
        try {
            return objClazz.getConstructor(parameterTypes).newInstance(parameters);
        } catch (Exception e) {
            throw new ConfigurationException("Fails to create " + className + " instance", e);
        }
    }
    
    /**
     * <p>
     * Gets the property value of the specified name. If the property value does not exist or is empty, a
     * ConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name.
     *
     * @throws ConfigurationException if the property value does not exist or is empty.
     */
    private String getPropertyValue(Property property, String name)
    throws ConfigurationException {
        String value = property.getValue(name);
        
        if (value == null) {
            throw new ConfigurationException("Missing value for property " + name);
        }
        if (value.trim().length() == 0) {
            throw new ConfigurationException("Empty value for property " + name);
        }
        return value;
    }
}
