/*
 * FileNameMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;

import java.io.File;


/**
 * <p>
 * FileNameMatcher is an abstract class that handles filename oriented matching logic.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileNameMatcher implements FileMatcher {
    /**
     * <p>
     * Default protected constructor to be accessed by subclass. Does nothing.
     * </p>
     */
    protected FileNameMatcher() {
    }

    /**
     * <p>
     * Determines if the file can be processed by the processor that this matcher is associated with.
     * </p>
     * 
     * <p>
     * This method just extracts the filename and passes it to the abstract match() method.
     * </p>
     *
     * @param file the file to match
     *
     * @return whether the file matches
     *
     * @throws NullPointerException if file is null
     * @throws FileMatchingException if error occurs in file matching
     */
    public boolean match(File file) throws FileMatchingException {
        if (file == null) {
            throw new NullPointerException("file is null");
        }
        return match(file.getName());
    }

    /**
     * <p>
     * Determines whether the file matches based on the given filename. Implementations should define the logic to
     * match the filename.
     * </p>
     * 
     * <p>
     * This method assumes that the filename argument will never be null. I/O errors should not occur here since the
     * matching logic is filename oriented.
     * </p>
     *
     * @param filename the filename to match
     *
     * @return whether the filename matches
     *
     * @throws FileMatchingException if error occurs in filename matching
     */
    protected abstract boolean match(String filename) throws FileMatchingException;
}
