/*
 * MatcherUtil.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.Property;

import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * A utility class for the common operations shared by the file matchers, mainly in the constructors. This class and
 * all its methods are package-private, so that they can only be used by classes in the same package.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
class MatcherUtil {
    /**
     * <p>
     * Private constructor to prevent outside instantiation.
     * </p>
     */
    private MatcherUtil() {
    }

    /**
     * <p>
     * Gets the property values under the specified name. If the property is missing or it does not hold any values, or
     * if canBeEmpty flag is false but any property value is empty, a ConfigurationException will be thrown.
     * </p>
     * 
     * <p>
     * This method is best used in the constructor that accepts a Property instance to configure the matcher. The name
     * argument should not be null.
     * </p>
     *
     * @param property the property to get the values from
     * @param name the property name
     * @param canBeEmpty whether the property values can be empty
     *
     * @return the property values under the specified name
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if the property is missing or it does not hold any values, or if canBeEmpty flag
     *         is false but any property value is empty
     */
    static String[] getPropertyValues(Property property, String name, boolean canBeEmpty)
        throws ConfigurationException
    {
        if (property == null) {
            throw new NullPointerException("property is null");
        }

        // load the values of the property
        String[] values = property.getValues(name);

        if (values == null) {
            throw new ConfigurationException("Missing values for property " + name);
        }

        // check if any property value is empty
        if (!canBeEmpty) {
            for (int i = 0; i < values.length; i++) {
                if (values[i].trim().length() == 0) {
                    throw new ConfigurationException("Property value contains empty String");
                }
            }
        }
        return values;
    }

    /**
     * <p>
     * Simply checks if the list argument is non-null and non-empty.
     * </p>
     *
     * @param list the list to check
     *
     * @throws NullPointerException if list is null
     * @throws IllegalArgumentException if list is empty
     */
    static void checkNonEmptyList(List list) {
        if (list == null) {
            throw new NullPointerException("list is null");
        }
        if (list.isEmpty()) {
            throw new IllegalArgumentException("list is empty");
        }
    }

    /**
     * <p>
     * Checks if the list argument is non-null and non-empty, and that the list only contains elements of the specified
     * class.
     * </p>
     * 
     * <p>
     * This method is best used in the constructor that accepts a List instance to configure the matcher. The clazz
     * argument should not be null.
     * </p>
     *
     * @param list the list to check
     * @param clazz the class of the element
     *
     * @throws NullPointerException if list is null
     * @throws IllegalArgumentException if list is empty, or contains element not of the specified class
     */
    static void checkListForObject(List list, Class clazz) {
        checkNonEmptyList(list);

        for (Iterator i = list.iterator(); i.hasNext();) {
            if (!clazz.isInstance(i.next())) {
                throw new IllegalArgumentException("list contains element not of type " + clazz.getName());
            }
        }
    }

    /**
     * <p>
     * Checks if the list argument is non-null and non-empty, and that the list only contains elements of type String.
     * If canBeEmpty flag is false but any String element is empty, an IllegalArgumentException will be thrown.
     * </p>
     * 
     * <p>
     * This method is best used in the constructor that accepts a List of Strings to configure the matcher.
     * </p>
     *
     * @param list the list to check
     * @param canBeEmpty whether the String elements can be empty
     *
     * @throws NullPointerException if list is null
     * @throws IllegalArgumentException if list is empty, or contains element not of type String, or if canBeEmpty flag
     *         is false but any String element is empty
     */
    static void checkListForString(List list, boolean canBeEmpty) {
        checkNonEmptyList(list);

        for (Iterator i = list.iterator(); i.hasNext();) {
            Object obj = i.next();

            if (!(obj instanceof String)) {
                throw new IllegalArgumentException("list contains element not of type String");
            }
            if (!canBeEmpty && (((String) obj).trim().length() == 0)) {
                throw new IllegalArgumentException("list contains empty String");
            }
        }
    }
}
