/*
 * MimeMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;

import com.topcoder.util.config.Property;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * <p>
 * Concrete implementation of the FileTypeMatcher class that matches file types with one or more mimes. The mime of the
 * file type is configured in Magic Numbers component. The matching is case-sensitive.
 * </p>
 * 
 * <p>
 * To follow the reflection contract, this class can be constructed via a Property instance. The supported property is
 * named "mimes" which is potentially multi-valued.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcher extends FileTypeMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file mimes to match. Empty mimes are not allowed even if they are valid
     * in the Magic Numbers component.
     * </p>
     */
    public static final String MIMES_PROPERTY_KEY = "mimes";

    /**
     * <p>
     * A set of file mimes (String) to match. This set is initialized in constructor and is immutable.
     * </p>
     */
    private Set mimes = null;

    /**
     * <p>
     * Constructor with single file mime. The mime cannot be empty.
     * </p>
     *
     * @param mime the single file mime to match
     *
     * @throws NullPointerException if mime is null
     * @throws IllegalArgumentException if mime is empty
     */
    public MimeMatcher(String mime) {
        if (mime == null) {
            throw new NullPointerException("mime is null");
        }
        if (mime.trim().length() == 0) {
            throw new IllegalArgumentException("mime is empty");
        }

        // create a Set to hold the single mime
        mimes = new HashSet();
        mimes.add(mime);
    }

    /**
     * <p>
     * Constructor with multiple file mimes. The mime element cannot be empty String. The class will store a copy of
     * the list.
     * </p>
     *
     * @param mimes the multiple file mimes to match
     *
     * @throws NullPointerException if mimes is null
     * @throws IllegalArgumentException if mimes is empty or contains non-String instance, or contains empty String
     */
    public MimeMatcher(List mimes) {
        MatcherUtil.checkListForString(mimes, false);

        // store the mimes in a set
        this.mimes = new HashSet(mimes);
    }

    /**
     * <p>
     * Constructor that loads mimes from a property. It should look for a "mimes" sub-property and extract multiple
     * values. The property value cannot be empty.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if the "mimes" sub-property is missing or it does not hold any values, or any
     *         property value is empty
     */
    public MimeMatcher(Property property) throws ConfigurationException {
        String[] mimes = MatcherUtil.getPropertyValues(property, MIMES_PROPERTY_KEY, false);

        // store the mimes in a set
        this.mimes = new HashSet(Arrays.asList(mimes));
    }

    /**
     * <p>
     * Checks whether the file type matches any of the mimes defined in the matcher. The matching is case-sensitive.
     * </p>
     *
     * @param type the file type to match
     *
     * @return whether the file type matches
     */
    protected boolean match(FileType type) {
        String mime = type.getMime();

        // look for the mime in a case-sensitive way
        return mimes.contains(mime);
    }
}
