/*
 * FileProcessorBase.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.processor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;

import com.topcoder.util.config.Property;


/**
 * <p>
 * A base implementation that can be used for FileProcessors. It provides alias and matcher support, including loading
 * them from configuration file. It serves TextLineCounter and CStyleLineCounter as well as can be generically used by
 * future implementations.
 * </p>
 * 
 * <p>
 * This class can load configuration from a Property. The following properties are supported:
 * 
 * <ul>
 * <li>
 * "alias" string value of the alias for this processor. This property is required and can not be empty.
 * </li>
 * <li>
 * "matcher" which specifies a file matcher. It contains a "classname" which specifies the fully qualified class name
 * of a FileMatcher implementation that owns a constructor with a Property argument. If not specified AnyFileMatcher
 * is used.
 * </li>
 * </ul>
 * </p>
 * 
 * <p>
 * This class is immutable and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileProcessorBase implements FileProcessor {
    /**
     * <p>
     * This property holds the alias for the file processor. It is required and can not be empty.
     * </p>
     */
    public static final String ALIAS_PROPERTY_KEY = "alias";

    /**
     * <p>
     * This property contains sub-properties to specify the attached FileMatcher implementation. When the
     * implementation is instantiated the property itself is passed in as constructor argument. If not specified
     * AnyFileMatcher is used.
     * </p>
     */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";

    /**
     * <p>
     * This is a sub-property in "matcher" to specify the fully qualified class name of the FileMatcher implementation
     * that owns a constructor with a Property argument.
     * </p>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /**
     * <p>
     * This is the alias for the file processor. It is set in constructor and never changed.
     * </p>
     */
    private String alias = null;

    /**
     * <p>
     * This is the file matcher for the file processor (and determines if a file can/should be processed). It is set in
     * constructor and never changed.
     * </p>
     */
    private FileMatcher matcher = null;

    /**
     * <p>
     * Constructor with the given alias and file matcher. It is protected for subclass to access.
     * </p>
     *
     * @param alias the alias for the file processor
     * @param matcher the file matcher associated with the file processor
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty
     */
    protected FileProcessorBase(String alias, FileMatcher matcher) {
        if (alias == null) {
            throw new NullPointerException("alias is null");
        }
        if (matcher == null) {
            throw new NullPointerException("matcher is null");
        }
        if (alias.trim().length() == 0) {
            throw new IllegalArgumentException("alias is empty");
        }
        this.alias = alias;
        this.matcher = matcher;
    }

    /**
     * <p>
     * This constructor loads configuration from the property. It is protected for subclass to access.
     * </p>
     * 
     * <p>
     * The non-empty alias is required, the matcher is optional. Any configuration-related error will be wrapped in a
     * ConfigurationException.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if any error occurs in configuring the processor
     */
    protected FileProcessorBase(Property property) throws ConfigurationException {
        if (property == null) {
            throw new NullPointerException("property is null");
        }

        // alias property is required and must be non-empty
        alias = getPropertyValue(property, ALIAS_PROPERTY_KEY);

        // matcher property is optional
        Property matcher = property.getProperty(FILE_MATCHER_PROPERTY_KEY);

        if (matcher != null) {
            // instantiate the FileMatcher class if specified
            this.matcher = (FileMatcher) configPropertyObject(matcher, CLASSNAME_PROPERTY_KEY, FileMatcher.class);
        } else {
            // otherwise, AnyFileMatcher is used
            this.matcher = new AnyFileMatcher();
        }
    }

    /**
     * <p>
     * Gets the alias of the processor.
     * </p>
     *
     * @return the alias of the file processor
     */
    public String getAlias() {
        return alias;
    }

    /**
     * <p>
     * Gets the file matcher associated with the processor.
     * </p>
     *
     * @return a file matcher associated with the processor
     */
    public FileMatcher getFileMatcher() {
        return matcher;
    }

    /**
     * <p>
     * Instantiates the class based on the information collected from the specified property, using a Property
     * argument. The class name is specified under the property of the given name. The property should be a node
     * representing the class in the configuration file. The class to be instantiated must be of type clazz.
     * </p>
     *
     * @param property a node representing the class.
     * @param name the name of the property specifying the class name.
     * @param clazz the superclass or superinterface of the instantiated object.
     *
     * @return an instance of the specified class.
     *
     * @throws ConfigurationException if a problem occurs while configuring, or required properties are missing or
     *         empty.
     */
    private Object configPropertyObject(Property property, String name, Class clazz)
        throws ConfigurationException
    {
        String className = getPropertyValue(property, name);

        // locate the class for instantiation
        Class objClazz = null;

        try {
            objClazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new ConfigurationException("Unable to locate the class " + className, e);
        }

        if (!clazz.isAssignableFrom(objClazz)) {
            throw new ConfigurationException("Class " + className + " is not of type " + clazz.getName());
        }

        // instantiate the class with a Property argument
        Class[] parameterTypes = new Class[] {Property.class};
        Object[] parameters = new Object[] {property};

        try {
            return objClazz.getConstructor(parameterTypes).newInstance(parameters);
        } catch (Exception e) {
            throw new ConfigurationException("Fails to create " + className + " instance", e);
        }
    }

    /**
     * <p>
     * Gets the property value of the specified name. If the property value does not exist or is empty, a
     * ConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name.
     *
     * @throws ConfigurationException if the property value does not exist or is empty.
     */
    private String getPropertyValue(Property property, String name)
        throws ConfigurationException
    {
        String value = property.getValue(name);

        if (value == null) {
            throw new ConfigurationException("Missing value for property " + name);
        }
        if (value.trim().length() == 0) {
            throw new ConfigurationException("Empty value for property " + name);
        }
        return value;
    }
}
