/*
 * AbstractStatsReporting.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;


/**
 * <p>
 * This class provides a StatsReporting implementation base. It aggregates results from each processor and classifies
 * them based on single file, directory or alias. It also provides an overall statistical result. It defers the actual
 * rendering logic to concrete implementations.
 * </p>
 * 
 * <p>
 * If base directory is specified, the file paths in the file stats and directory paths in the directory stats are all
 * relative to the base directory. Otherwise, the paths will be absolute.
 * </p>
 * 
 * <p>
 * Since version 2.0c it updates the signature of aggregate() and supports custom path separator other than
 * the default path separator in order to distinguish archive path from normal path. The following property is supported:
 *
 * <ul>
 * <li>
 * "path_separators" which contains zero, one or more custom path separators. It is optional. Default is "&gt;".
 * It's user's responsibility not to use a valid character in potential file names.
 * </li>
 * </ul>
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 */
public abstract class AbstractStatsReporting implements StatsReporting {
    /**
 * <p>
 * This property specifies one or more custom path separators to identify in addition to the default path separator.
 * It is optional. But if it exists the value should contain no duplicates. It's user's responsibility not to use a valid
 * character in potential file names.
 * </p>
 * 
 * @since 2.0c
 */
    public static final String PATH_SEPARATORS_PROPERTY_KEY = "path_separators";

    /**
     * <p>
     * The stats mapped to each file. The file path is relative to base directory or absolute if no base directory. Its
     * reference never changes.
     * </p>
     */
    private StatsCollection fileStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each directory. The directory path is relative to base directory or absolute if no base
     * directory. Its reference never changes.
     * </p>
     */
    private StatsCollection directoryStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each alias. Its reference never changes.
     * </p>
     */
    private StatsCollection aliasStats = new StatsCollection();

    /**
     * <p>
     * The overall statistical result. It is created anew when reporting resets.
     * </p>
     */
    private StatsResult overallStats = new StatsResult();

    /**
     * <p>
     * The base directory for the reporting. This variable is nullable (meaning all paths are absolute).
     * </p>
     */
    private File baseDirectory = null;
    
    /**
 * <p>
 * This field contains one or more path separators (each of which is a char). It is initialized to ">" and has both
 * getter and setter.
 * </p>
 * 
 * @since 2.0c
 */
    private String pathSeparators = ">";

    /**
     * <p>
     * Default protected constructor to be accessed by subclass. Does nothing.
     * </p>
     */
    protected AbstractStatsReporting() {
    }

    /**
     * <p>
     * Protected constructor with property to be accessed by subclass. Since version 2.0c this constructor loads the
     * optional "path_separators".
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if configuration error occurs
     */
    protected AbstractStatsReporting(Property property)
        throws ConfigurationException
    {
        if (property == null) {
            throw new NullPointerException("property is null");
        }
        
        // path separators is optional
        String pathSeparators = property.getValue(PATH_SEPARATORS_PROPERTY_KEY);
        
        if (pathSeparators != null) {
            try {
                setPathSeparators(pathSeparators);
            } catch (IllegalArgumentException e) {
                throw new ConfigurationException("Duplicate characters found in property: "
                        + PATH_SEPARATORS_PROPERTY_KEY);
            }
        }
    }

    /**
     * <p>
     * Aggregates a result into the reporting against file, each path it is in, alias and the overall statistical
     * result.
     * </p>
     * 
     * <p>
     * If base directory is specified, the file path and directory path are relative to base directory. Otherwise, the
     * paths are absolute. Notice the directory stats does not need to include the base directory, since it's the same
     * as overall.
     * </p>
     *
     * @param processor the processor that generated the result
     * @param filePath the file that was processed
     * @param result the processing result
     *
     * @throws NullPointerException if any argument is null
     * @throws IllegalArgumentException if filePath is empty 
     * @throws StatsReportingException if baseDirectory is not null but file is not in base directory, or if error
     *         occurs in aggregation
     */
    public void aggregate(FileProcessor processor, String filePath, StatsResult result)
        throws StatsReportingException
    {
        if (processor == null) {
            throw new NullPointerException("processor is null");
        }
        if (filePath == null) {
            throw new NullPointerException("file is null");
        }
        if (result == null) {
            throw new NullPointerException("result is null");
        }
        if (filePath.trim().length() == 0) {
            throw new IllegalArgumentException("filePath is empty");
        }

        if (baseDirectory != null) {
            String basePath = getCanonicalPath(baseDirectory);

            // include separator character in the base path to be stripped
            if (!basePath.endsWith(File.separator)) {
                basePath += File.separator;
            }

            // check if file is in base directory
            if (!filePath.startsWith(basePath)) {
                throw new StatsReportingException("file " + filePath + " is not in base directory " + basePath);
            }

            // strip the base path off the file path
            filePath = filePath.substring(basePath.length());
        }

        // aggregate the stats for each path it is in
        StringBuffer sb = new StringBuffer(filePath);
        int index = -1;

        while ((index = lastIndexOfSeparator(sb.toString())) >= 0) {
            String path = sb.substring(0, index);
            
            // aggregate the file stats for file path
            if (sb.charAt(index) != File.separatorChar) {
                fileStats.add(path, result);
                continue;
            }

            // add back the separator character for root directory
            if ((baseDirectory == null) && (path.indexOf(File.separator) < 0)) {
                path += File.separator;
            }

            // include everything after the base directory as the path
            directoryStats.add(path, result);

            // proceed with the parent directory
            sb.setLength(index);
        }

        // aggregate the alias stats
        aliasStats.add(processor.getAlias(), result);

        // aggregate the overall statistical result
        overallStats.aggregate(result);
    }

    /**
     * <p>
     * Resets the reporting such that all the results are cleared. Base directory can be null when absolute paths are
     * desirable.
     * </p>
     *
     * @param baseDirectory the base directory to use, can be null
     *
     * @throws IllegalArgumentException if baseDirectory is not null but is not a valid directory
     */
    public void reset(File baseDirectory) {
        if ((baseDirectory != null) && !baseDirectory.isDirectory()) {
            throw new IllegalArgumentException("baseDirectory is not a valid directory");
        }

        // clear the results
        fileStats.clear();
        directoryStats.clear();
        aliasStats.clear();
        overallStats = new StatsResult();

        // set the base directory
        this.baseDirectory = baseDirectory;
    }

    /**
     * <p>
     * Gets the file stats kept in this reporting.
     * </p>
     *
     * @return the file stats
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <p>
     * Gets the directory stats kept in this reporting.
     * </p>
     *
     * @return the directory stats
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <p>
     * Gets the alias stats kept in this reporting.
     * </p>
     *
     * @return the alias stats
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <p>
     * Gets the overall statistical result kept in this reporting. Returns a clone of the inner result.
     * </p>
     *
     * @return the overall statistical result
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <p>
     * Gets the base directory for the reporting. Can be null if all paths are absolute.
     * </p>
     *
     * @return the base directory, can be null
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }
    
    /**
 * <p>
 * Gets the custom path separators (that do not include the default path separator in the system).
 * </p>
 * 
 * 
 * @return the custom path separators
 * @since 2.0c
 */
    public String getPathSeparators() {        
        return pathSeparators;
    } 

/**
 * <p>
 * Sets the custom path separators, which should contain no duplicates (but can be empty).
 * </p>
 * 
 * 
 * @param pathSeparators the path separators to set
 * @throws NullPointerException if pathSeparators is null
 * @throws IllegalArgumentException if pathSeparators contains duplicates
 * @since 2.0c
 */
    public void setPathSeparators(String pathSeparators) {        
        if (pathSeparators == null) {
            throw new NullPointerException("pathSeparators is null");
        }
        
        // check for duplicates
        Set set = new HashSet();

        for (int i = 0; i < pathSeparators.length(); i++) {
            if (!set.add(new Character(pathSeparators.charAt(i)))) {
                throw new IllegalArgumentException("Duplicate characters found in path separators");
            }
        }
        
        // everything is fine here
        this.pathSeparators = pathSeparators;
    } 

    /**
     * <p>
     * Returns the canonical pathname string of the specified file. This method conveniently wraps any IOException into
     * StatsReportingException.
     * </p>
     *
     * @param file the file to get the canonical pathname string from
     *
     * @return the canonical pathname string
     *
     * @throws StatsReportingException if error occurs in getting the canonical pathname string
     */
    private String getCanonicalPath(File file) throws StatsReportingException {
        try {
            return file.getCanonicalPath();
        } catch (IOException e) {
            throw new StatsReportingException("Fails to get the canonical pathname string", e);
        }
    }
    
    /**
     * <p>Returns the last index of any of the separator characters in the specified path. If not found, -1 is
     * returned.
     * </p>
     *
     * @param path the path to find the separator characters
     * @return the last index of any of the separator characters, or -1 if not found
     */
    private int lastIndexOfSeparator(String path) {
        for (int i = path.length() - 1; i >= 0; i--) {
            char c = path.charAt(i);
            
            if (c == File.separatorChar || pathSeparators.indexOf(c) >= 0) {
                return i;
            }
        }
        return -1;
    }
}
