/*
 * StatsCollection.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.StatsResult;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * StatsCollection is a generic container for mapping from String keys to StatsResult instances. It is used in
 * AbstractStatsReporting to hold file, directory and alias classified mappings. The class provides basic manipulation
 * methods for the mapping.
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollection {
    /**
     * <p>
     * Mapping from String to StatsResult instances. This is used to classify file, directory and alias based stats in
     * AbstractStatsReporting. It is initialized in constructor and the reference is never changed.
     * </p>
     */
    private Map stats = null;

    /**
     * <p>
     * Default constructor. The collection is empty.
     * </p>
     */
    public StatsCollection() {
        stats = new HashMap();
    }

    /**
     * <p>
     * Adds a result to the specified key.
     * </p>
     * 
     * <p>
     * If mapping already exists, it should aggregate the result with this one. Otherwise saves a clone of the result
     * to the mapping.
     * </p>
     *
     * @param key the key to associate the result with
     * @param stats the result to add
     *
     * @throws NullPointerException if key or stats is null
     * @throws IllegalArgumentException if key is empty
     */
    public void add(String key, StatsResult stats) {
        if (stats == null) {
            throw new NullPointerException("stats is null");
        }
        checkString(key);

        StatsResult result = (StatsResult) this.stats.get(key);

        if (result == null) {
            // if mapping does not exist, save a clone of the result
            this.stats.put(key, new StatsResult(stats));
        } else {
            // otherwise, just aggregate the result
            result.aggregate(stats);
        }
    }

    /**
     * <p>
     * Gets the result associated with a key. The result is cloned before return. If no result is associated return
     * null.
     * </p>
     *
     * @param key the key to retrieve
     *
     * @return the result associated with the key, or null if no result is associated
     *
     * @throws NullPointerException if key is null
     * @throws IllegalArgumentException if key is empty
     */
    public StatsResult get(String key) {
        checkString(key);

        StatsResult result = (StatsResult) stats.get(key);

        if (result == null) {
            // if no result is found, return null
            return null;
        } else {
            // otherwise, return the cloned result
            return new StatsResult(result);
        }
    }

    /**
     * <p>
     * Gets the list of keys in the collection. Should return a copy and sort the strings in alphabetical order (case
     * insensitive). This defines the listing order in the statistical report.
     * </p>
     *
     * @return the list of keys in the collection
     */
    public List getAllKeys() {
        // copy from the key set
        List keys = new ArrayList(stats.keySet());

        // sort the strings in case-insensitive alphabetical order
        Collections.sort(keys, String.CASE_INSENSITIVE_ORDER);

        return keys;
    }

    /**
     * <p>
     * Clears all the results in the collection.
     * </p>
     */
    public void clear() {
        stats.clear();
    }

    /**
     * <p>
     * Checks if the given string is null or empty.
     * </p>
     *
     * @param str the string to check against.
     *
     * @throws NullPointerException if str is null.
     * @throws IllegalArgumentException if str is empty.
     */
    private void checkString(String str) {
        if (str == null) {
            throw new NullPointerException("String argument is null");
        }
        if (str.trim().length() == 0) {
            throw new IllegalArgumentException("String argument is empty");
        }
    }
}
