/*
 * ExtensionMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * Unit tests for ExtensionMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcherTest extends TestCase {
    /**
     * The files to match against by ExtensionMatcher instances.
     */
    private static final String[] FILES = {"test_files/a/p.txt", "test_files/a/q.java", "test_files/b/test.sql"};

    /**
     * The ExtensionMatcher instances to test against.
     */
    private ExtensionMatcher[] matchers = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ExtensionMatcherTest.class);

        return suite;
    }

    /**
     * Prepares three ExtensionMatcher instances for testing. matchers[0] is constructed with a single extension "txt".
     * matchers[1] is constructed with multiple extensions "TXT", "JAVA". matchers[2] is constructed with property
     * loaded with extensions "txt", "java".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        matchers = new ExtensionMatcher[3];
        matchers[0] = new ExtensionMatcher("txt");
        matchers[1] = new ExtensionMatcher(Arrays.asList(new String[] {"TXT", "JAVA"}));
        matchers[2] = new ExtensionMatcher(ConfigHelper.getRootProperty(ConfigHelper.GOOD_EXTENSION_MATCHER_NAMESPACE));
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null extension. Expects NullPointerException.
     */
    public void testConstructor_NullExtension() {
        try {
            new ExtensionMatcher((String) null);
            fail("Creates ExtensionMatcher with null extension");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with null extensions. Expects NullPointerException.
     */
    public void testConstructor_NullExtensions() {
        try {
            new ExtensionMatcher((List) null);
            fail("Creates ExtensionMatcher with null extensions");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty extensions. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyExtensions() {
        try {
            new ExtensionMatcher(new ArrayList());
            fail("Creates ExtensionMatcher with empty extensions");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with extensions containing non-String instance. Expects IllegalArgumentException.
     */
    public void testConstructor_NonStringExtensions() {
        List extensions = new ArrayList();

        // add a non-String element
        extensions.add(new Object());

        try {
            new ExtensionMatcher(extensions);
            fail("Creates ExtensionMatcher with extensions containing non-String instance");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new ExtensionMatcher((Property) null);
            fail("Creates ExtensionMatcher with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no value is present). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new ExtensionMatcher(ConfigHelper.getRootProperty(ConfigHelper.BAD_EXTENSION_MATCHER_NAMESPACE));
            fail("Creates ExtensionMatcher with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of match method with the three files. Expects results for matchers[0]: true, false, false. Expects results
     * for matchers[1]: true, true, false. Expects results for matchers[2]: true, true, false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch() throws Exception {
        File[] files = new File[FILES.length];

        for (int i = 0; i < FILES.length; i++) {
            files[i] = new File(FILES[i]);
        }

        // test with matchers[0] - only matches txt file
        assertTrue("Fails to match file", matchers[0].match(files[0]));
        assertFalse("Fails to match file", matchers[0].match(files[1]));
        assertFalse("Fails to match file", matchers[0].match(files[2]));

        // test with matchers[1] - matches txt, java files
        assertTrue("Fails to match file", matchers[1].match(files[0]));
        assertTrue("Fails to match file", matchers[1].match(files[1]));
        assertFalse("Fails to match file", matchers[1].match(files[2]));

        // test with matchers[2] - matches txt, java files
        assertTrue("Fails to match file", matchers[2].match(files[0]));
        assertTrue("Fails to match file", matchers[2].match(files[1]));
        assertFalse("Fails to match file", matchers[2].match(files[2]));
    }
}
